@@ -14,9 +14,11 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.Vector;
 
 import org.eclipse.cdt.core.CCorePlugin;
@@ -27,10 +29,19 @@ import org.eclipse.cdt.core.IMarkerGenerator;
 import org.eclipse.cdt.core.model.ICModelMarker;
 import org.eclipse.cdt.core.resources.ACBuilder;
 import org.eclipse.cdt.core.resources.IConsole;
-import org.eclipse.cdt.managedbuilder.core.IConfiguration; 
+import org.eclipse.cdt.managedbuilder.buildmodel.BuildDescriptionManager;
+import org.eclipse.cdt.managedbuilder.buildmodel.IBuildCommand;
+import org.eclipse.cdt.managedbuilder.buildmodel.IBuildDescription;
+import org.eclipse.cdt.managedbuilder.buildmodel.IBuildIOType;
+import org.eclipse.cdt.managedbuilder.buildmodel.IBuildResource;
+import org.eclipse.cdt.managedbuilder.buildmodel.IBuildStep;
+import org.eclipse.cdt.managedbuilder.core.IConfiguration;
 import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
 import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
 import org.eclipse.cdt.managedbuilder.envvar.IBuildEnvironmentVariable;
+import org.eclipse.cdt.managedbuilder.internal.buildmodel.DescriptionBuilder;
+import org.eclipse.cdt.managedbuilder.internal.buildmodel.IBuildModelBuilder;
+import org.eclipse.cdt.managedbuilder.internal.buildmodel.StepBuilder;
 import org.eclipse.cdt.managedbuilder.macros.BuildMacroException;
 import org.eclipse.cdt.managedbuilder.macros.IBuildMacroProvider;
 import org.eclipse.cdt.managedbuilder.makegen.IManagedBuilderMakefileGenerator;
@@ -61,7 +72,6 @@ import org.eclipse.core.runtime.SubProgressMonitor;
  * @since 1.2 
  */
 public class GeneratedMakefileBuilder extends ACBuilder {
-	
 	/**
 	 * @since 1.2
 	 */
@@ -205,6 +215,81 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 		}
 	}
 
+	private static class OtherConfigVerifier implements IResourceDeltaVisitor {
+		IConfiguration config;
+		IConfiguration configs[];
+		Configuration otherConfigs[];
+		int resourceChangeState;
+		
+		private static final IPath[] ignoreList = {
+			new Path(".cdtproject"), //$NON-NLS-1$
+			new Path(".cproject"), //$NON-NLS-1$
+			new Path(".cdtbuild"), //$NON-NLS-1$
+			new Path(".settings"), //$NON-NLS-1$
+		};
+		
+		OtherConfigVerifier(IConfiguration cfg){
+			config = cfg;
+			configs = cfg.getManagedProject().getConfigurations();
+			otherConfigs = new Configuration[configs.length - 1];
+			int counter = 0;
+			for(int i = 0; i < configs.length; i++){
+				if(configs[i] != config)
+					otherConfigs[counter++] = (Configuration)configs[i];
+			}
+		}
+		
+		public boolean visit(IResourceDelta delta) throws CoreException {
+			
+			IResource rc = delta.getResource();
+			if(rc.getType() == IResource.FILE){
+				if(isResourceValuable(rc))
+					resourceChangeState |= delta.getKind();
+				return false;
+			}
+			return !isGeneratedForConfig(rc, config) && isResourceValuable(rc);
+		}
+		
+		public void updateOtherConfigs(IResourceDelta delta){
+			if(delta == null)
+				resourceChangeState = ~0;
+			else {
+				try {
+					delta.accept(this);
+				} catch (CoreException e) {
+					resourceChangeState = ~0;
+				}
+			}
+				
+			setResourceChangeStateForOtherConfigs();
+		}
+		
+		private void setResourceChangeStateForOtherConfigs(){
+			for(int i = 0; i < otherConfigs.length; i++){
+				otherConfigs[i].addResourceChangeState(resourceChangeState);
+			}
+		}
+	
+		private boolean isGeneratedForConfig(IResource resource, IConfiguration cfg) {
+			// Is this a generated directory ...
+			IPath path = resource.getProjectRelativePath();
+			IPath root = new Path(cfg.getName());
+			// It is if it is a root of the resource pathname
+			if (root.isPrefixOf(path))
+				return true;
+			return false;
+		}
+
+		private boolean isResourceValuable(IResource rc){
+			IPath path = rc.getProjectRelativePath();
+			for(int i = 0; i < ignoreList.length; i++){
+				if(ignoreList[i].equals(path))
+					return false;
+			}
+			return true;
+		}
+	}
+
 	// String constants
 	private static final String BUILD_ERROR = "ManagedMakeBuilder.message.error";	//$NON-NLS-1$
 	private static final String BUILD_FINISHED = "ManagedMakeBuilder.message.finished";	//$NON-NLS-1$
@@ -219,15 +304,23 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 	private static final String TRACE_FOOTER = "]: ";	//$NON-NLS-1$
 	private static final String TRACE_HEADER = "GeneratedmakefileBuilder trace [";	//$NON-NLS-1$
 	private static final String TYPE_CLEAN = "ManagedMakeBuilder.type.clean";	//$NON-NLS-1$
-	private static final String TYPE_FULL = "ManagedMakeBuilder.type.full";	//$NON-NLS-1$
 	private static final String TYPE_INC = "ManagedMakeBuider.type.incremental";	//$NON-NLS-1$
 	private static final String WARNING_UNSUPPORTED_CONFIGURATION = "ManagedMakeBuilder.warning.unsupported.configuration";	//$NON-NLS-1$
+	private static final String BUILD_CANCELLED = "ManagedMakeBuilder.message.cancelled";	//$NON-NLS-1$
+	private static final String BUILD_FINISHED_WITH_ERRS = "ManagedMakeBuilder.message.finished.with.errs";	//$NON-NLS-1$
+	private static final String BUILD_FAILED_ERR = "ManagedMakeBuilder.message.internal.builder.error";	//$NON-NLS-1$
+	private static final String BUILD_STOPPED_ERR = "ManagedMakeBuilder.message.stopped.error";	//$NON-NLS-1$
+	private static final String INTERNAL_BUILDER_HEADER_NOTE = "ManagedMakeBuilder.message.internal.builder.header.note";	//$NON-NLS-1$
+	private static final String TYPE_REBUILD = "ManagedMakeBuider.type.rebuild";	//$NON-NLS-1$
+	private static final String INTERNAL_BUILDER = "ManagedMakeBuilder.message.internal.builder";	//$NON-NLS-1$
 	public static boolean VERBOSE = false;
 	
 	// Local variables
 	protected Vector generationProblems;
 	protected IProject[] referencedProjects;
 	protected List resourcesToBuild;
+	private IConsole console;
+	private ConsoleOutputStream consoleOutStream;
 	public static void outputTrace(String resourceName, String message) {
 		if (VERBOSE) {
 			System.out.println(TRACE_HEADER + resourceName + TRACE_FOOTER + message + NEWLINE);
@@ -276,9 +369,7 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 			ConsoleOutputStream consoleOutStream = console.getOutputStream();
 			// Report a successful clean
 			String[] consoleHeader = new String[3];
-			if (buildType == FULL_BUILD) {
-			    consoleHeader[0] = ManagedMakeMessages.getResourceString(TYPE_FULL);
-			} else if (buildType == INCREMENTAL_BUILD) {
+			if (buildType == FULL_BUILD || buildType == INCREMENTAL_BUILD) {
 				consoleHeader[0] = ManagedMakeMessages.getResourceString(TYPE_INC);
 			} else {
 				consoleHeader[0] = new String();
@@ -330,13 +421,83 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 			outputError(getProject().getName(), "Build information is not valid");	//$NON-NLS-1$
 			return referencedProjects;
 		}
-		
+
+		IConfiguration cfg = info.getDefaultConfiguration();
+
+		updateOtherConfigs(cfg, kind);
+
+		if(((Configuration)cfg).isInternalBuilderEnabled()){
+			invokeInternalBuilder(cfg, kind != FULL_BUILD, ((Configuration)cfg).getInternalBuilderIgnoreErr(), monitor);
+
+			// Scrub the build info the project
+			info.setRebuildState(false);
+			return referencedProjects;
+		}
+
 		// Create a makefile generator for the build
 		IManagedBuilderMakefileGenerator generator = ManagedBuildManager.getBuildfileGenerator(info.getDefaultConfiguration());
 		generator.initialize(getProject(), info, monitor);
 
+		//perform necessary cleaning and build type calculation
+		if(cfg.needsFullRebuild()){
+			//configuration rebuild state is set to true,
+			//full rebuild is needed in any case
+			//clean first, then make a full build
+			outputTrace(getProject().getName(), "config rebuild state is set to true, making a full rebuild");	//$NON-NLS-1$
+			clean(new SubProgressMonitor(monitor, IProgressMonitor.UNKNOWN));
+			fullBuild(info, generator, monitor);
+		} else {
+			boolean fullBuildNeeded = info.needsRebuild();
+			IBuildDescription des = null;
+			
+			IResourceDelta delta = kind == FULL_BUILD ? null : getDelta(getProject());
+			if(delta == null)
+				fullBuildNeeded = true;
+			if(cfg.needsRebuild() || delta != null){
+				//use a build desacription model to calculate the resources to be cleaned 
+				//only in case there are some changes to the project sources or build information
+				try{
+					int flags = BuildDescriptionManager.REBUILD | BuildDescriptionManager.DEPS_DEPFILE_INFO;
+					if(delta != null)
+						flags |= BuildDescriptionManager.REMOVED;
+
+					outputTrace(getProject().getName(), "using a build description..");	//$NON-NLS-1$
+
+					des = BuildDescriptionManager.createBuildDescription(info.getDefaultConfiguration(), getDelta(getProject()), flags);
+	
+					BuildDescriptionManager.cleanGeneratedRebuildResources(des);
+				} catch (Throwable e){
+					//TODO: log error
+					outputError(getProject().getName(), "error occured while build description calculation: " + e.getLocalizedMessage());	//$NON-NLS-1$
+					//in case an error occured, make it behave in the old stile:
+					if(info.needsRebuild()){
+						//make a full clean if an info needs a rebuild
+						clean(new SubProgressMonitor(monitor, IProgressMonitor.UNKNOWN));
+						fullBuildNeeded = true;
+					}
+					else if(delta != null && !fullBuildNeeded){
+						// Create a delta visitor to detect the build type
+						ResourceDeltaVisitor visitor = new ResourceDeltaVisitor(info);
+						delta.accept(visitor);
+						if (visitor.shouldBuildFull()) {
+							clean(new SubProgressMonitor(monitor, IProgressMonitor.UNKNOWN));
+							fullBuildNeeded = true;
+						}
+					}
+				}
+			}
+
+			if(fullBuildNeeded){
+				outputTrace(getProject().getName(), "performing a full build");	//$NON-NLS-1$
+				fullBuild(info, generator, monitor);
+			} else {
+				outputTrace(getProject().getName(), "performing an incremental build");	//$NON-NLS-1$
+				incrementalBuild(delta, info, generator, monitor);
+			}
+		}
+/*
 		// So let's figure out why we got called
-		if (kind == FULL_BUILD || info.needsRebuild()) {
+		if (kind == FULL_BUILD) {
 			outputTrace(getProject().getName(), "Full build needed/requested");	//$NON-NLS-1$
 			fullBuild(info, generator, monitor);
 		}
@@ -388,12 +549,17 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 				}
 			}
 		}
+*/
 		// Scrub the build info the project
 		info.setRebuildState(false);
 		// Ask build mechanism to compute deltas for project dependencies next time
 		return referencedProjects;
 	}
 
+	private void updateOtherConfigs(IConfiguration cfg, int buildKind){
+		new OtherConfigVerifier(cfg).updateOtherConfigs(buildKind == FULL_BUILD ? null : getDelta(getProject()));
+	}
+	
 	/**
 	 * Check whether the build has been canceled. Cancellation requests 
 	 * propagated to the caller by throwing <code>OperationCanceledException</code>.
@@ -508,7 +674,7 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 		//in case one or more of the generated makefiles (e.g. dep files) are corrupted, 
 		//the builder invocation might fail because of the possible syntax errors, so e.g. "make clean" will not work 
 		//we need to explicitly clean the generated directories
-		clean(new SubProgressMonitor(monitor, IProgressMonitor.UNKNOWN));
+//		clean(new SubProgressMonitor(monitor, IProgressMonitor.UNKNOWN));
 		
 		// Regenerate the makefiles for this project
 		checkCancel(monitor);
@@ -595,7 +761,6 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 				args.add("clean");	//$NON-NLS-1$
 				break;
 			case FULL_BUILD:
-				args.add("clean");	//$NON-NLS-1$
 			case INCREMENTAL_BUILD:
 				args.add("all");	//$NON-NLS-1$
 				break;
@@ -749,8 +914,6 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 				String[] consoleHeader = new String[3];
 				switch (buildType) {
 					case FULL_BUILD:
-						consoleHeader[0] = ManagedMakeMessages.getResourceString(TYPE_FULL);
-						break;
 					case INCREMENTAL_BUILD:
 						consoleHeader[0] = ManagedMakeMessages.getResourceString(TYPE_INC);
 						break;
@@ -889,7 +1052,7 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 						break;
 					}
 					case FULL_BUILD: {
-						makeArgs.add("clean"); //$NON-NLS-1$        
+//						makeArgs.add("clean"); //$NON-NLS-1$        
 						makeArgs.add("pre-build"); //$NON-NLS-1$ 
 						makeArgs.add("main-build"); //$NON-NLS-1$                           
 						break;
@@ -919,10 +1082,18 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 						} catch (IOException e) {
 						}
 
-						if (launcher.waitAndRead(epm.getOutputStream(), epm.getOutputStream(),
+						int state = launcher.waitAndRead(epm.getOutputStream(), epm.getOutputStream(),
 								new SubProgressMonitor(monitor,
-										IProgressMonitor.UNKNOWN)) != CommandLauncher.OK) {
+										IProgressMonitor.UNKNOWN));
+						if(state != CommandLauncher.OK){
 							errMsg = launcher.getErrorMessage();
+							
+							if(state == CommandLauncher.COMMAND_CANCELED){
+								//TODO: the better way of handling cancel is needed
+								//currently the rebuild state is set to true forcing the full rebuild
+								//on the next builder invocation
+								info.getDefaultConfiguration().setRebuildState(true);
+							}
 						}
 
 						// Force a resync of the projects without allowing the user to cancel. 
@@ -1005,4 +1176,488 @@ public class GeneratedMakefileBuilder extends ACBuilder {
 			}
 		}
 	}
+	
+	/**
+	 * called to invoke the MBS Internal Builder for building the given configuration
+	 *  
+	 * @param cfg configuration to be built
+	 * @param buildIncrementaly if true, incremental build will be performed,
+	 * only files that need rebuild will be built.
+	 * If false, full rebuild will be performed
+	 * @param resumeOnErr if true, build will continue in case of error while building.
+	 * If false the build will stop on the first error 
+	 * @param monitor monitor
+	 */
+	protected void invokeInternalBuilder(IConfiguration cfg, 
+			boolean buildIncrementaly,
+			boolean resumeOnErr,
+			IProgressMonitor monitor) {
+		// Get the project and make sure there's a monitor to cancel the build
+		IProject currentProject = cfg.getOwner().getProject();
+		if (monitor == null) {
+			monitor = new NullProgressMonitor();
+		}
+		
+		String[] msgs = new String[2];
+		msgs[0] = ManagedMakeMessages.getResourceString(INTERNAL_BUILDER);
+		msgs[1] = currentProject.getName();
+
+		monitor.beginTask("", 1000);	//$NON-NLS-1$
+		monitor.subTask(ManagedMakeMessages.getFormattedString(MAKE, msgs));
+
+		ConsoleOutputStream consoleOutStream = null;
+		IConsole console = null;
+		OutputStream epmOutputStream = null;
+		try {
+			int flags = 0;
+			IResourceDelta delta = null;
+			
+			if(buildIncrementaly){
+				flags = BuildDescriptionManager.REBUILD | BuildDescriptionManager.REMOVED;
+				delta = getDelta(currentProject);
+			}
+			
+			IBuildDescription des = BuildDescriptionManager.createBuildDescription(cfg, delta, flags);
+			
+			DescriptionBuilder builder = new DescriptionBuilder(des, buildIncrementaly, resumeOnErr);
+
+			// Get a build console for the project
+			StringBuffer buf = new StringBuffer();
+			console = CCorePlugin.getDefault().getConsole();
+			console.start(currentProject);
+			consoleOutStream = console.getOutputStream();
+			String[] consoleHeader = new String[3];
+			if(buildIncrementaly)
+				consoleHeader[0] = ManagedMakeMessages.getResourceString(TYPE_INC);
+			else
+				consoleHeader[0] = ManagedMakeMessages.getResourceString(TYPE_REBUILD);
+						
+			consoleHeader[1] = cfg.getName();
+			consoleHeader[2] = currentProject.getName();
+			buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+			buf.append(ManagedMakeMessages.getFormattedString(CONSOLE_HEADER, consoleHeader));
+			buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+			buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+			
+			buf.append(ManagedMakeMessages.getResourceString(INTERNAL_BUILDER_HEADER_NOTE));
+			buf.append("\n"); //$NON-NLS-1$
+			
+			if(!cfg.isSupported()){
+				buf.append(ManagedMakeMessages.getFormattedString(WARNING_UNSUPPORTED_CONFIGURATION,new String[] {cfg.getName(),cfg.getToolChain().getName()}));
+				buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+				buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+			}
+			consoleOutStream.write(buf.toString().getBytes());
+			consoleOutStream.flush();
+			
+			if(builder.getNumCommands() > 0) {
+				// Remove all markers for this project
+				removeAllMarkers(currentProject);
+				
+				// Hook up an error parser manager
+				String[] errorParsers = cfg.getErrorParserList(); 
+				ErrorParserManager epm = new ErrorParserManager(getProject(), des.getDefaultBuildDirLocation(), this, errorParsers);
+				epm.setOutputStream(consoleOutStream);
+				// This variable is necessary to ensure that the EPM stream stay open
+				// until we explicitly close it. See bug#123302.
+				epmOutputStream = epm.getOutputStream();
+				
+				int status = builder.build(epmOutputStream, epmOutputStream, new SubProgressMonitor(monitor, 1000));
+	
+				// Report either the success or failure of our mission 
+				buf = new StringBuffer();
+				
+				switch(status){
+				case IBuildModelBuilder.STATUS_OK:
+					buf.append(ManagedMakeMessages
+							.getFormattedString(BUILD_FINISHED,
+									currentProject.getName()));
+					break;
+				case IBuildModelBuilder.STATUS_CANCELLED:
+					buf.append(ManagedMakeMessages
+							.getResourceString(BUILD_CANCELLED));
+					break;
+				case IBuildModelBuilder.STATUS_ERROR_BUILD:
+					String msg = resumeOnErr ? 
+							ManagedMakeMessages.getResourceString(BUILD_FINISHED_WITH_ERRS) :
+								ManagedMakeMessages.getResourceString(BUILD_STOPPED_ERR);
+					buf.append(msg);
+					break;
+				case IBuildModelBuilder.STATUS_ERROR_LAUNCH:
+				default:
+					buf.append(ManagedMakeMessages
+							.getResourceString(BUILD_FAILED_ERR));
+					break;
+				}
+				buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
+	
+				// Write message on the console 
+				consoleOutStream.write(buf.toString().getBytes());
+				consoleOutStream.flush();
+				epmOutputStream.close();
+				epmOutputStream = null;
+				// Generate any error markers that the build has discovered 
+				monitor.subTask(ManagedMakeMessages
+						.getResourceString(MARKERS));
+				addBuilderMarkers(epm);
+				epm.reportProblems();
+			} else {
+				buf = new StringBuffer();
+				buf.append(ManagedMakeMessages.getFormattedString(NOTHING_BUILT, getProject().getName()));
+				buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
+				consoleOutStream.write(buf.toString().getBytes());
+				consoleOutStream.flush();
+			}
+
+		} catch (Exception e) {
+			if(consoleOutStream != null){
+				StringBuffer buf = new StringBuffer();
+				String errorDesc = ManagedMakeMessages
+							.getResourceString(BUILD_ERROR);
+				buf.append(errorDesc);
+				buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
+				buf.append("(").append(e.getLocalizedMessage()).append(")"); //$NON-NLS-1$ //$NON-NLS-2$
+				buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
+
+				try {
+					consoleOutStream.write(buf.toString().getBytes());
+					consoleOutStream.flush();
+				} catch (IOException e1) {
+				}
+			}
+			forgetLastBuiltState();
+		} finally {
+			if(epmOutputStream != null){
+				try {
+					epmOutputStream.close();
+				} catch (IOException e) {
+				} 
+			}
+			if(consoleOutStream != null){
+				try {
+					consoleOutStream.close();
+				} catch (IOException e) {
+				}
+			}
+			getGenerationProblems().clear();
+			monitor.done();
+		}
+	}
+	
+	/**
+	 * Called to invoke the MBS Internal Builder for building the given resources in
+	 * the given configuration
+	 * 
+	 * This method is considered experimental.  Clients implementing this API should expect
+	 * possible changes in the API.
+	 *  
+	 * @param cfg configuration to be built
+	 * @param buildIncrementaly if true, incremental build will be performed,
+	 * only files that need rebuild will be built.
+	 * If false, full rebuild will be performed
+	 * @param resumeOnErr if true, build will continue in case of error while building.
+	 * If false the build will stop on the first error 
+	 * @param monitor Progress monitor.  For every resource built this monitor will consume one unit of work.
+	 */
+	public void invokeInternalBuilder(IResource[] resourcesToBuild, IConfiguration cfg, 
+			boolean buildIncrementaly,
+			boolean resumeOnErr,
+			boolean initNewConsole,
+			boolean printFinishedMessage,
+			IProgressMonitor monitor) {
+		// Get the project and make sure there's a monitor to cancel the build
+		IProject currentProject = cfg.getOwner().getProject();
+		if (monitor == null) {
+			monitor = new NullProgressMonitor();
+		}
+		
+		try {
+			int flags = 0;
+			IResourceDelta delta = null;
+			
+			if(buildIncrementaly){
+				flags = BuildDescriptionManager.REBUILD | BuildDescriptionManager.REMOVED;
+				delta = getDelta(currentProject);
+			}
+			
+			
+			String[] msgs = new String[2];
+			msgs[0] = ManagedMakeMessages.getResourceString(INTERNAL_BUILDER);
+			msgs[1] = currentProject.getName();
+
+			if(initNewConsole)
+				initNewBuildConsole(currentProject);
+			
+			StringBuffer buf = new StringBuffer();
+			
+			if (initNewConsole) {
+				if (buildIncrementaly)
+					buf.append(ManagedMakeMessages.getResourceString("GeneratedMakefileBuilder.buildSelectedIncremental")); //$NON-NLS-1$
+				else
+					buf.append(ManagedMakeMessages.getResourceString("GeneratedMakefileBuilder.buildSelectedRebuild")); //$NON-NLS-1$
+				
+
+				buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$
+				buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$
+
+				buf.append(ManagedMakeMessages
+						.getResourceString(INTERNAL_BUILDER_HEADER_NOTE));
+				buf.append("\n"); //$NON-NLS-1$
+			}
+			
+			
+			if(!cfg.isSupported()){
+				buf.append(ManagedMakeMessages.getFormattedString(WARNING_UNSUPPORTED_CONFIGURATION,new String[] {cfg.getName(),cfg.getToolChain().getName()}));
+				buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+				buf.append(System.getProperty("line.separator", "\n"));	//$NON-NLS-1$	//$NON-NLS-2$
+			}
+			consoleOutStream.write(buf.toString().getBytes());
+			consoleOutStream.flush();
+				
+			// Remove all markers for this project
+			// TODO remove only necessary markers
+			removeAllMarkers(currentProject);
+			
+			IBuildDescription des = BuildDescriptionManager.createBuildDescription(cfg, delta, flags);
+			
+			// Hook up an error parser manager
+			String[] errorParsers = cfg.getErrorParserList(); 
+			ErrorParserManager epm = new ErrorParserManager(currentProject, des.getDefaultBuildDirLocation(), this, errorParsers);
+			epm.setOutputStream(consoleOutStream);
+			// This variable is necessary to ensure that the EPM stream stay open
+			// until we explicitly close it. See bug#123302.
+			OutputStream epmOutputStream = epm.getOutputStream();
+			
+			boolean errorsFound = false;
+			
+		doneBuild: for (int k = 0; k < resourcesToBuild.length; k++) {
+				IBuildResource buildResource = des
+						.getBuildResource(resourcesToBuild[k]);
+
+//				step collector 
+				Set dependentSteps = new HashSet();
+
+//				get dependent IO types
+				IBuildIOType depTypes[] = buildResource.getDependentIOTypes();
+
+//				iterate through each type and add the step the type belongs to to the collector
+				for(int j = 0; j < depTypes.length; j++){
+				IBuildIOType type = depTypes[j];
+				if(type != null && type.getStep() != null)
+					dependentSteps.add(type.getStep());
+				}
+
+				monitor.subTask(ManagedMakeMessages.getResourceString("GeneratedMakefileBuilder.buildingFile") + resourcesToBuild[k].getProjectRelativePath()); //$NON-NLS-1$
+				
+				// iterate through all build steps
+				Iterator stepIter = dependentSteps.iterator();
+				
+				while(stepIter.hasNext())
+				{
+					IBuildStep step = (IBuildStep) stepIter.next();
+					
+					StepBuilder stepBuilder = new StepBuilder(step);
+					
+					int status = stepBuilder.build(consoleOutStream, epmOutputStream, new SubProgressMonitor(monitor, 1, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK));
+					
+					// Refresh the output resource without allowing the user to cancel. 
+					// This is probably unkind, but short of this there is no way to ensure 
+					// the UI is up-to-date with the build results 
+					IBuildIOType[] outputIOTypes = step.getOutputIOTypes();
+					
+					for(int j = 0; j < outputIOTypes.length; j++ )
+					{
+						IBuildResource[] resources = outputIOTypes[j].getResources();
+						
+						for(int i = 0; i < resources.length; i++)
+						{
+							IFile file = currentProject.getFile(resources[i].getLocation());
+							file.refreshLocal(IResource.DEPTH_INFINITE, null);
+						}
+					}
+					
+					// check status
+					
+					switch (status) {
+					case IBuildModelBuilder.STATUS_OK:
+						// don't print anything if the step was successful,
+						// since the build might not be done as a whole
+						break;
+					case IBuildModelBuilder.STATUS_CANCELLED:
+						buf.append(ManagedMakeMessages
+								.getResourceString(BUILD_CANCELLED));
+						break doneBuild;
+					case IBuildModelBuilder.STATUS_ERROR_BUILD:
+						errorsFound = true;
+						if (!resumeOnErr) {
+							buf.append(ManagedMakeMessages
+									.getResourceString(BUILD_STOPPED_ERR));
+							break doneBuild;
+						}
+						break;
+					case IBuildModelBuilder.STATUS_ERROR_LAUNCH:
+					default:
+						buf.append(ManagedMakeMessages
+								.getResourceString(BUILD_FAILED_ERR));
+						break doneBuild;
+					}
+				}
+
+				
+			}
+
+			// check status
+			// Report either the success or failure of our mission
+			buf = new StringBuffer();
+
+			
+			buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
+			
+			if (printFinishedMessage) {
+				if (errorsFound) {
+					buf.append(ManagedMakeMessages
+							.getResourceString(BUILD_FAILED_ERR));
+				} else {
+					buf
+							.append(ManagedMakeMessages
+									.getResourceString("GeneratedMakefileBuilder.buildResourcesFinished")); //$NON-NLS-1$
+				}
+			}
+			
+			// Write message on the console 
+			consoleOutStream.write(buf.toString().getBytes());
+			consoleOutStream.flush();
+			epmOutputStream.close();
+
+			// Generate any error markers that the build has discovered 
+			addBuilderMarkers(epm);
+			epm.reportProblems();
+			consoleOutStream.close();
+		} catch (Exception e) {
+			StringBuffer buf = new StringBuffer();
+			String errorDesc = ManagedMakeMessages
+						.getResourceString(BUILD_ERROR);
+			buf.append(errorDesc);
+			buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
+			buf.append("(").append(e.getLocalizedMessage()).append(")"); //$NON-NLS-1$ //$NON-NLS-2$ 
+
+			forgetLastBuiltState();
+		} finally {
+			getGenerationProblems().clear();
+		}
+	}
+	
+	private void removeAllMarkers(IFile file) {
+		IMarker[] markers;
+		try {
+			markers = file.findMarkers(
+					ICModelMarker.C_MODEL_PROBLEM_MARKER, true,
+					IResource.DEPTH_INFINITE);
+		} catch (CoreException e) {
+			// Handled just about every case in the sanity check
+			return;
+		}
+		if (markers != null) {
+			try {
+				file.getWorkspace().deleteMarkers(markers);
+			} catch (CoreException e) {
+				// The only situation that might cause this is some sort of
+				// resource change event
+				return;
+			}
+		}
+	}
+	
+	public void cleanFile(IFile file, IProgressMonitor monitor) {
+
+		monitor.subTask("Cleaning output file(s) for "
+				+ file.getProjectRelativePath());
+
+		// remove all markers on the file
+		removeAllMarkers(file);
+
+		IProject currentProject = file.getProject();
+
+		IManagedBuildInfo info = ManagedBuildManager
+				.getBuildInfo(currentProject);
+
+		// if we have no info then don't do anything
+		if (info == null) {
+			// monitor.worked(1);
+			return;
+
+		}
+
+		IConfiguration cfg = info.getDefaultConfiguration();
+
+		// figure out the output file for this file
+		IPath sourcePath = file.getProjectRelativePath();
+
+		int flags = BuildDescriptionManager.REBUILD | BuildDescriptionManager.REMOVED;;
+		IResourceDelta delta = getDelta(currentProject);
+		
+		try {
+			IBuildDescription des = BuildDescriptionManager
+					.createBuildDescription(cfg, delta, flags);
+
+			IBuildResource buildResource = des.getBuildResource(file);
+
+			// step collector
+			Set dependentSteps = new HashSet();
+
+			// get dependent IO types
+			IBuildIOType depTypes[] = buildResource.getDependentIOTypes();
+
+			// iterate through each type and add the step the type belongs to to
+			// the collector
+			for (int j = 0; j < depTypes.length; j++) {
+				IBuildIOType type = depTypes[j];
+				if (type != null && type.getStep() != null)
+					dependentSteps.add(type.getStep());
+			}
+
+			// iterate through all build steps
+			Iterator stepIter = dependentSteps.iterator();
+
+			while (stepIter.hasNext()) {
+				IBuildStep step = (IBuildStep) stepIter.next();
+
+				// Delete the output resources
+				IBuildIOType[] outputIOTypes = step.getOutputIOTypes();
+
+				for (int j = 0; j < outputIOTypes.length; j++) {
+					IBuildResource[] resources = outputIOTypes[j]
+							.getResources();
+
+					for (int i = 0; i < resources.length; i++) {
+						IResource outputFile = currentProject
+								.findMember(resources[i].getFullPath().removeFirstSegments(1)); // strip project name
+
+						if (outputFile != null)
+							outputFile.delete(true, new SubProgressMonitor(
+									monitor, 1));
+					}
+				}
+
+			}
+
+		} catch (CoreException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		}
+		
+	}
+
+
+	/**
+	 * @param currentProject
+	 * @return
+	 * @throws CoreException
+	 */
+	private void initNewBuildConsole(IProject currentProject) throws CoreException {
+		// Get a build console for the project
+		console = CCorePlugin.getDefault().getConsole();
+		console.start(currentProject);
+		consoleOutStream = console.getOutputStream();
+	}
 }
